@@ -17,7 +17,7 @@ use Sub::Util 'subname', 'set_subname';
 use namespace::clean;
 
 # ABSTRACT: Compile a parse tree into perl code
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 *_clean_eval= *Language::FormulaEngine::Compiler::_CleanEval::_clean_eval;
 
@@ -126,7 +126,7 @@ sub _get_perl_generator {
 	my $fqn= subname($info->{native}) || '';
 	# For security, make reasonably sure that perl will parse the subname as a function name.
 	# This regex is more restrictive than perl's actual allowed identifier names.
-	$fqn =~ /^[A-Za-z_][A-Za-z0-9_]*::([A-Za-z0-9_]+::)*\w+$/u
+	$fqn =~ /^[A-Za-z_][A-Za-z0-9_]*::([A-Za-z0-9_]+::)*\p{Word}+$/
 		or die "Can't compile function '$name'; native function does not have a valid fully qualified name '$fqn'\n";
 	# Create a generator that injects this function name
 	return sub {
@@ -171,7 +171,7 @@ Language::FormulaEngine::Compiler - Compile a parse tree into perl code
 
 =head1 VERSION
 
-version 0.01
+version 0.02
 
 =head1 DESCRIPTION
 
